#include "mainwindow.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent),
      model(nullptr),
      active_model(nullptr),
      xml_view_empty(true),
      xml_active_view_empty(true)
{
    this->resize(800, 600);
    QWidget* central_widget = new QWidget(this);
    main_layout = new QVBoxLayout(central_widget);

    file_dialog = new QFileDialog(this);
    file_dialog->setFileMode(QFileDialog::ExistingFile);
    file_dialog->setViewMode(QFileDialog::Detail);
    file_dialog->setNameFilter("XML (*.xml)");

    menu = new QMenu(tr("Меню"), this);

    auto exit_action = make_action(tr("Выход"), tr("Закрыть программу"), SLOT(on_exit_action_clicked()));
    auto fullscreen_action = make_action(tr("Во весь экран"), tr("Открыть окно во весь экран/Выйти из полноэкранного режима"),
                                         SLOT(on_fullscreen_action_clicked()));
    exit_action->setShortcut(Qt::Key_Escape);
    fullscreen_action->setShortcut(Qt::Key_F11);
    exit_action->setShortcutContext(Qt::ApplicationShortcut);
    fullscreen_action->setShortcutContext(Qt::ApplicationShortcut);

    menu->addAction(exit_action);
    menu->addAction(fullscreen_action);
    menu->addAction(make_action(tr("Открыть"), tr("Загрузка xml-файла данных"), SLOT(on_open_action_clicked())));
    menu->addAction(make_action(tr("Закрыть всё"), tr("Очистить область вывода данных"), SLOT(on_close_action_clicked())));
    menu->addAction(make_action(tr("Переключить все файлы"), tr("Показать/спрятать окно со всеми открытыми файлами"),
                                SLOT(on_toggle_all_action_clicked())));
    menu->addAction(make_action(tr("Переключить активные файлы"), tr("Показать/спрятать окно с активынми файлами"),
                                SLOT(on_toggle_active_action_clicked())));

    this->statusBar();

    xml_view = new QTreeView(this);
    xml_active_view = new QTreeView(this);
    xml_view->header()->setStretchLastSection(false);
    xml_active_view->header()->setStretchLastSection(false);
    xml_view->header()->setSectionResizeMode(QHeaderView::Stretch);
    xml_active_view->header()->setSectionResizeMode(QHeaderView::Stretch);
    xml_view->setContextMenuPolicy(Qt::ActionsContextMenu);

    context_menu = new QMenu(xml_view);

    context_menu->addAction(make_action(tr("Сделать активным"), tr("Добавить файл в \"Активные файлы\""),
                                        SLOT(on_make_active_action_clicked()), false));
    context_menu->addAction(make_action(tr("Закрыть"), tr("Закрыть файл"),
                                        SLOT(on_remove_action_clicked()), false));

    xml_view->setContextMenuPolicy(Qt::CustomContextMenu);

    connect(xml_view, SIGNAL(customContextMenuRequested(QPoint)), this,
            SLOT(on_context_menu_requested(QPoint)));

    xml_view->setFont(QFont("Calibri", 10));
    xml_active_view->setFont(QFont("Calibri", 12));
    menuBar()->setFont(QFont("Calibri", 12));
    statusBar()->setFont(QFont("Calibri", 11));

    this->menuBar()->addMenu(menu);

    view_label = new QLabel(this);
    active_view_label = new QLabel(this);
    view_label->setFont(QFont("Calibri", 14));
    active_view_label->setFont(QFont("Calibri", 14));
    view_label->setText(tr("Все файлы"));
    active_view_label->setFont(QFont("Calibri", 14));
    active_view_label->setText(tr("Активные файлы"));

    main_layout->addWidget(view_label);
    main_layout->addWidget(xml_view, 1);
    main_layout->addWidget(active_view_label);
    main_layout->addWidget(xml_active_view, 2);

    central_widget->setLayout(main_layout);

    this->setCentralWidget(central_widget);
}

MainWindow::~MainWindow()
{
}

QAction* MainWindow::make_action(const QString& p_name, const QString& p_tip, const char* p_slot, bool status_bar){
    QAction* action = new QAction(p_name, this);
    connect(action, SIGNAL(triggered()), this, p_slot);
    action->setFont(QFont("Calibri", 10));
    if(status_bar)
        action->setStatusTip(p_tip);
    else
        action->setWhatsThis(p_tip);
    return action;
};

void MainWindow::on_open_action_clicked(){
    QStringList filenames = QFileDialog::getOpenFileNames(this, tr("Open File"), "/home",
                                                          tr("XML files (*.xml)"));
    if(xml_view_empty){
        model = new XMLParsingModel(filenames, this);
        xml_view_empty = false;
    }else{
        xml_view->reset();
        model->add_file(filenames);
    }
    xml_view->setModel(model);
}

void MainWindow::on_close_action_clicked(){
    xml_view->reset();
    xml_active_view->reset();
    delete model;
    delete active_model;
    xml_active_view_empty = true;
    xml_view_empty = true;
}

void MainWindow::on_exit_action_clicked(){
    this->close();
}

void MainWindow::on_fullscreen_action_clicked()
{
    if(isFullScreen())
        showNormal();
    else
        showFullScreen();
}

void MainWindow::on_make_active_action_clicked()
{
    auto filename = last_context_index.sibling(last_context_index.row(), 0).data().toString();
    if(xml_active_view_empty){
        active_model = new XMLParsingModel(QStringList(model->get_path(filename)), this);
        xml_active_view_empty = false;
    }else{
        xml_active_view->reset();
        active_model->add_file(QStringList(model->get_path(filename)));
    }
    xml_active_view->setModel(active_model);
}

void MainWindow::on_remove_action_clicked()
{
    xml_view->reset();
    model->delete_file(last_context_index);
    xml_view->setModel(model);
}

void MainWindow::on_toggle_active_action_clicked()
{
    if(xml_active_view->isHidden()){
        xml_active_view->show();
        active_view_label->show();
    }else{
        xml_active_view->hide();
        active_view_label->hide();
    }
}

void MainWindow::on_toggle_all_action_clicked()
{
    if(xml_view->isHidden()){
        xml_view->show();
        view_label->show();
    }else{
        xml_view->hide();
        view_label->hide();
    }
}

void MainWindow::on_context_menu_requested(QPoint point)
{
    QModelIndex index = xml_view->indexAt(point);
    if(index.isValid()){
        last_context_index = index;
        context_menu->exec(xml_view->viewport()->mapToGlobal(point));
    }
}

